# Copyright (C) 2014-2015, Availab.io(R) Ltd. All rights reserved.
from jinja2 import Environment
import json


def substitute_variables_recursively(something, variables):
    """
    Recursively substitutes variables in strings, arrays and dictionaries.

    If you try to pass any other object it will be returned unharmed.
    """
    env = Environment()

    if isinstance(something, str) or isinstance(something, unicode):
        return env.from_string(something).render(**variables)
    elif isinstance(something, list):
        new_list = []
        for itm in something:
            new_list.append(substitute_variables_recursively(
                itm, variables)
            )
        return new_list
    elif isinstance(something, dict):
        new_dict = {}
        for key, value in something.iteritems():
            new_key = env.from_string(key).render(**variables)
            new_value = substitute_variables_recursively(value, variables)
            new_dict[new_key] = new_value
        return new_dict
    else:
        return something



from __future__ import print_function
import sys

verbosity = 1

class Colors(object):
    HEADER = '\033[95m'
    BLUE = '\033[94m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    CYAN = '\033[36m'
    RED = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

    RED_BG = '\033[41m'
    GREEN_BG = '\033[42m'
    YELLOW_BG = '\033[43m'
    BLUE_BG = '\033[44m'
    MAGENTA_BG = '\033[45m'


def eprint(to_print):
    """Print to stderr, regardless on verbose status"""
    print(
        u"{0}{1}{2}".format(Colors.RED, to_print, Colors.ENDC), file=sys.stderr
    )

def vprint(needed_verbosity, color, to_print, allow_line_update=False):
    """
    Print to stdout if verbosity is enough to do so.
    """
    if verbosity >= needed_verbosity:

        if color is not None:
            to_print = u"{0}{1}{2}".format(color, to_print, Colors.ENDC)

        if allow_line_update:
            print(u""+to_print, end="")
            sys.stdout.flush()
        else:
            print(u""+to_print)




class ApiliskException(Exception):
    """
    Vase class for all apilisk-related exceptions
    """

class ObjectNotFound(ApiliskException):
    """
    Requested object (project, testcase) has not been found
    """



#!/usr/bin/env python
import json
import argparse
from os.path import expanduser

import apilisk.printer
from apilisk.runner import Runner
from apilisk.exceptions import ApiliskException
from apilisk.printer import eprint, vprint
from apilisk.apiwatcher_client import ApiwatcherClient
from apilisk.junit_formatter import JunitFormatter

def _check_args(args):
    if args.action == "init":
        # Must all be filled
        if (
            args.client_id is None or
            args.agent_id is None or
            args.client_secret is None
        ):
            eprint("Options --client-id, --client-secret and --agent-id must "
                "be set for init action."
            )
            exit(1)

    elif args.action == "run":
        cfg = {}
        if (args.client_id is not None or args.client_secret is not None
        ):
            cfg = _get_config_data(
                args.client_id, args.client_secret, args.agent_id
            )
        else:
            try:
                with open(args.config_file) as cfg_file:
                    cfg = json.load(cfg_file)
            except IOError as e:
                eprint("Could not open configuration file at {0}: {1}".format(
                    args.config_file, e.message
                ))
                exit(1)

        if args.project is None:
            eprint("Project hash (-p) is mandatory for action 'run'")
            exit(1)

        return cfg

    else:
        eprint(
            u"Unknown action {0}, allowed values are run or init".format(
            args.action
        ))
        parser.print_usage()
        exit(1)


def _get_config_data(client_id, client_secret, name):
    return {
        "host": "https://api2.apiwatcher.com",
        "port": 443,
        "client_id": client_id,
        "client_secret": client_secret,
        "agent_id": name
    }

def _create_config_file(data, filename):
    with open(filename, 'w') as outfile:
        json.dump(data, outfile)

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "action", default=None,
        help="What to do - init or run."
    )
    parser.add_argument(
        "--client-id", default=None, type=str,
        help="Client id for init"
    )
    parser.add_argument(
        "--client-secret", default=None, type=str,
        help="Client id for init"
    )
    parser.add_argument(
        "--agent-id", default="NOT SET", type=str,
        help="Agent id for init"
    )
    parser.add_argument(
        "-p", "--project", default=None,
        help="Hash of the project."
    )
    parser.add_argument(
        "-d", "--dataset", type=int,
        help="Id of dataset to use. None if no dataset should be used."
    )
    parser.add_argument(
        "-v", "--verbose", type=int, default=1,
        help="0 = no output, 1 = what is being done, 2 = data"
    )
    parser.add_argument(
        "-c", "--config-file",
        default="{0}/.apilisk.json".format(expanduser("~")),
        help="Path to configuration file."
    )
    parser.add_argument(
        "-j", "--junit", help="Provide output in junit format.",
        action="store_true"
    )
    parser.add_argument(
        "-o", "--junit-output-file", help="Path to junit output file",
        type=str, default="./output.xml"
    )
    parser.add_argument(
        "-i", "--include-data", help="Insert data into results.",
        action="store_true"
    )
    parser.add_argument(
        "-u", "--upload", help="Upload data to platform.",
        action="store_true"
    )

    args = parser.parse_args()
    apilisk.printer.verbosity = args.verbose

    cfg = _check_args(args)

    # Switch according to action
    if args.action == "init":
        _create_config_file(
            _get_config_data(
                args.client_id, args.client_secret, args.agent_id
            ),
            args.config_file
        )
    elif args.action == "run":
        try:
            client = ApiwatcherClient(cfg)
            project_cfg = client.get_project_config(args.project)

            runner = Runner(project_cfg, args.dataset)
            results = runner.run_project(
                include_data=args.include_data, debug=True
            )

            if args.junit:
                fmt = JunitFormatter(project_cfg, results)
                fmt.to_file("./output.xml")

            if args.upload:
                client.upload_results(project_cfg, results)

        except ApiliskException as e:
            eprint(e.message)
            exit(1)



"""
Client for apiwatcher platform
"""
from requests.exceptions import ConnectionError

from apiwatcher_pyclient.client import Client
from apiwatcher_pyclient.exceptions import ApiwatcherClientException
from apilisk.printer import vprint, Colors, eprint
from apilisk.exceptions import ObjectNotFound, ApiliskException

class ApiwatcherClient:
    """
    Client using apiwatcher-pyclient to communicate with the platform
    """

    def __init__(self, apilisk_cfg):
        """
        Initialize and log in to platform
        """
        try:
            self.agent_id = apilisk_cfg["agent_id"]
            self.client = Client(
                apilisk_cfg["host"], apilisk_cfg["port"],
                verify_certificate=True
            )

            vprint(
                1, None,
                "### Authorizing to {0} ... ".format(apilisk_cfg["host"]), True
            )

            self.client.authorize_client_credentials(
                apilisk_cfg["client_id"], apilisk_cfg["client_secret"],
                "private_agent"
            )
            vprint(
                1, Colors.GREEN,
                "\r### Authorizing to {0} ... OK".format(apilisk_cfg["host"])
            )

        except KeyError as e:
            raise ApiliskException(
                "Key {0} is missing in configuration file.".format(e.message)
            )
        except ApiwatcherClientException as e:
            raise ApiliskException(
                "Could not authenticate to Apiwatcher platform: {0}".format(
                    e.message
                )
            )
        except ConnectionError as e:
            raise ApiliskException(
                "Could not connect to Apiwatcher platform: {0}".format(
                    e.message
                )
            )


    def get_project_config(self, project_hash):
        """
        Return configuration of a project
        """
        vprint(
            1, None,
            "### Getting configuraton of project {0} ... ".format(project_hash),
            True
        )

        rsp = self.client.get(
            "/api/projects/{0}/configuration".format(project_hash)
        )
        if rsp.status_code == 404:
            raise ObjectNotFound(
                "Project with hash {0} has not been found".format(
                    project_hash
                )
            )
        elif rsp.status_code != 200:
            raise ApiliskException(
                "Could not get configuration of project {0}: {1}".format(
                    project_hash, rsp.json()["message"]
                )
            )

        vprint(
            1, Colors.GREEN,
            "\r### Getting configuraton of project {0} ... OK".format(project_hash)
        )
        cfg = rsp.json()["data"]
        vprint(
            2, Colors.GREEN,
            "### Summary: {0} testcases, {1} requests, {2} datasets".format(
                len(cfg["testcases"]), len(cfg["requests"]),
                len(cfg["datasets"])
            ))

        return cfg

    def upload_results(self, config, results):
        """Upload data to platform"""
        vprint(
            1, None,
            "### Uploading data to Apiwatcher platform ...", True
        )

        rsp = self.client.post(
            "/api/projects/{0}/remote-results".format(results["project_hash"]),
            data={
                "agent_id": self.agent_id,
                "configuration": config,
                "results": results
            }
        )

        if rsp.status_code == 201:
            vprint(
                1, Colors.GREEN,
                "\r### Uploading data to Apiwatcher platform ... OK"
            )
        else:
            eprint("### Upload failed with response code {0}".format(rsp.status_code))



import json
import copy
import pytz
import sys

from datetime import datetime

from apilisk.curl_caller import CurlCaller
from apilisk.printer import vprint, Colors
from apilisk.exceptions import ObjectNotFound, ApiliskException
from apiwatcher_pyclient.client import Client


class Runner(object):

    def __init__(self, project_cfg, dataset_id):
        """
        Initializes all the stuff
        """
        self.project_hash = project_cfg["project_hash"]
        self.project_name = project_cfg["project_name"]
        self.testcases = {
            str(item["id"]): item for item in project_cfg["testcases"]
        }
        self.requests = {
            str(item["id"]): item for item in project_cfg["requests"]
        }

        self.dataset = None
        if dataset_id is not None:
            for dts in project_cfg["datasets"]:
                if dts["id"] == dataset_id:
                 self.dataset = copy.deepcopy(dts)

            if self.dataset == None:
                raise ObjectNotFound(
                    u"Dataset with id {0} has not been found".format(
                        dataset_id
                    )
                )

    def run_project(self, debug=False, include_data=False):
        """
        Runs testcases from project one project
        """
        results = []

        time_start = datetime.now()

        total_count = len(self.testcases)
        success_count = 0
        failed_count = 0
        vprint(
            1, None,
            u"## Starting project {0} ({1})".format(
                self.project_name, self.project_hash
            )
        )
        for tc_id in self.testcases:
            res = self.run_one_testcase(tc_id, debug, include_data)
            if res["status"] == "success":
                success_count += 1
            else:
                failed_count += 1
            results.append(res)

        duration_sec = (datetime.now() - time_start).total_seconds()

        if failed_count > 0:
            vprint(
                1, Colors.RED,
                u"## Failed {0} testcases out of {1} in {2} sec.".format(
                    failed_count, total_count, duration_sec
                )
            )
        else:
            vprint(
                1, Colors.GREEN, u"## Success in {0} sec".format(duration_sec)
            )

        return {
            "project_hash": self.project_hash,
            "total_count": total_count,
            "success_count": success_count,
            "failed_count": failed_count,
            "duration_sec": duration_sec,
            "results": results
        }

    def run_one_testcase(self, tc_id, debug=False, include_data=False):
        """
        Runs a single testcase
        """
        # Merge dataset variables and request variables
        variables = {
            "var": copy.deepcopy(
                self.dataset["variables"]
            ) if self.dataset is not None else {},
            "req": []
        }

        auth = self.testcases[tc_id]["authentication"]
        status = "success"
        results = []

        time_start = datetime.now()

        vprint(
            1, None, u"# {0} ... ".format(
                self.testcases[tc_id]["name"]
            ), True
        )

        for step in self.testcases[tc_id]["steps"]:
            if step["action"] == "call_request":
                caller = CurlCaller(
                    step["data"], variables, authentication=auth, debug=debug
                )
                result, req_var = caller.handle_and_get_report(
                    include_data=include_data
                )
                variables["req"].append(req_var)
                results.append(result)

                if result["status"] == "failed":
                    status = "failed"
                    break


        if status == 'success':
            vprint(
                1, Colors.GREEN, u"\r# {0} ... SUCCESS".format(
                    self.testcases[tc_id]["name"]
                )
            )
        else:
            vprint(
                1, Colors.RED, u"\r# {0} ... FAILED".format(
                    self.testcases[tc_id]["name"]
                )
            )

        return {
            "testcase_id": int(tc_id),
            "steps_results": results,
            "status": status,
            "duration_sec": (datetime.now() - time_start).total_seconds()
        }
